# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .entity_edge_source_target import EntityEdgeSourceTarget
from .entity_property import EntityProperty


class EdgeType(UniversalBaseModel):
    description: str
    name: str
    properties: typing.Optional[typing.List[EntityProperty]] = None
    source_targets: typing.Optional[typing.List[EntityEdgeSourceTarget]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
