# This file was auto-generated by Fern from our API Definition.

import typing

Reranker = typing.Union[typing.Literal["rrf", "mmr", "node_distance", "episode_mentions", "cross_encoder"], typing.Any]
